@@ -124,11 +124,6 @@ class AddressList(MyTreeWidget):
             if addr_URL:
                 menu.addAction(_("View on block explorer"), lambda: webbrowser.open(addr_URL))
 
-        if any(not self.wallet.is_frozen(addr) for addr in addrs):
-            menu.addAction(_("Freeze"), lambda: self.parent.set_frozen_state(addrs, True))
-        if any(self.wallet.is_frozen(addr) for addr in addrs):
-            menu.addAction(_("Unfreeze"), lambda: self.parent.set_frozen_state(addrs, False))
-
         run_hook('receive_menu', menu, addrs, self.wallet)
         menu.exec_(self.viewport().mapToGlobal(position))
 
