@@ -3,43 +3,56 @@
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2012 thomasv@gitorious
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Permission is hereby granted, free of charge, to any person
+# obtaining a copy of this software and associated documentation files
+# (the "Software"), to deal in the Software without restriction,
+# including without limitation the rights to use, copy, modify, merge,
+# publish, distribute, sublicense, and/or sell copies of the Software,
+# and to permit persons to whom the Software is furnished to do so,
+# subject to the following conditions:
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
-
-import sys, time, datetime, re, threading
-from electrum.i18n import _
-from electrum.util import print_error, print_msg
-import os.path, json, ast, traceback
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
-from electrum import DEFAULT_PORTS
+
+from electrum.i18n import _
+from electrum.network import DEFAULT_PORTS
+from electrum.network import serialize_server, deserialize_server
 
 from util import *
 
-#protocol_names = ['TCP', 'HTTP', 'SSL', 'HTTPS']
-#protocol_letters = 'thsg'
 protocol_names = ['TCP', 'SSL']
 protocol_letters = 'ts'
 
-class NetworkDialog(QDialog):
+class NetworkDialog(WindowModalDialog):
     def __init__(self, network, config, parent):
-
-        QDialog.__init__(self,parent)
-        self.setModal(1)
-        self.setWindowTitle(_('Network'))
+        WindowModalDialog.__init__(self, parent, _('Network'))
         self.setMinimumSize(375, 20)
+        self.nlayout = NetworkChoiceLayout(network, config)
+        vbox = QVBoxLayout(self)
+        vbox.addLayout(self.nlayout.layout())
+        vbox.addLayout(Buttons(CancelButton(self), OkButton(self)))
 
+    def do_exec(self):
+        result = self.exec_()
+        if result:
+            self.nlayout.accept()
+        return result
+
+
+class NetworkChoiceLayout(object):
+    def __init__(self, network, config, wizard=False):
         self.network = network
         self.config = config
         self.protocol = None
@@ -49,7 +62,7 @@ class NetworkDialog(QDialog):
         if not proxy_config:
             proxy_config = { "mode":"none", "host":"localhost", "port":"9050"}
 
-        if parent:
+        if not wizard:
             n = len(network.get_interfaces())
             if n:
                 status = _("Blockchain") + ": " + "%d "%(network.get_local_height()) + _("blocks") +  ".\n" + _("Getting block headers from %d nodes.")%n
@@ -60,10 +73,9 @@ class NetworkDialog(QDialog):
             else:
                 status += "\n" + _("Disconnected from server")
         else:
-            status = _("Please choose a server.") + "\n" + _("Select 'Cancel' if you are offline.")
+            status = _("Please choose a server.") + "\n" + _("Press 'Next' if you are offline.")
 
         vbox = QVBoxLayout()
-        vbox.setSpacing(30)
         hbox = QHBoxLayout()
         l = QLabel()
         l.setPixmap(QPixmap(":icons/network.png"))
@@ -76,6 +88,7 @@ class NetworkDialog(QDialog):
             + _("This blockchain is used to verify the transactions sent by the address server.")
         hbox.addWidget(HelpButton(msg))
         vbox.addLayout(hbox)
+        vbox.addSpacing(15)
 
         # grid layout
         grid = QGridLayout()
@@ -87,38 +100,38 @@ class NetworkDialog(QDialog):
         self.server_host.setFixedWidth(200)
         self.server_port = QLineEdit()
         self.server_port.setFixedWidth(60)
+
         grid.addWidget(QLabel(_('Server') + ':'), 0, 0)
+        grid.addWidget(self.server_host, 0, 1, 1, 2)
+        grid.addWidget(self.server_port, 0, 3)
 
         # use SSL
         self.ssl_cb = QCheckBox(_('Use SSL'))
         self.ssl_cb.setChecked(auto_connect)
-        grid.addWidget(self.ssl_cb, 3, 1)
+        grid.addWidget(self.ssl_cb, 3, 1, 1, 3)
         self.ssl_cb.stateChanged.connect(self.change_protocol)
 
         # auto connect
-        self.autoconnect_cb = QCheckBox(_('Auto-connect'))
+        self.autoconnect_cb = QCheckBox(_('Select server automatically'))
         self.autoconnect_cb.setChecked(auto_connect)
-        grid.addWidget(self.autoconnect_cb, 0, 1)
+        grid.addWidget(self.autoconnect_cb, 1, 1, 1, 3)
         self.autoconnect_cb.setEnabled(self.config.is_modifiable('auto_connect'))
-        msg = _("If auto-connect is enabled, Electrum will always use a server that is on the longest blockchain.") + " " \
+        msg = _("If auto-connect is enabled, Electrum will always use a server that is on the longest blockchain.") + "\n" \
             + _("If it is disabled, Electrum will warn you if your server is lagging.")
-        grid.addWidget(HelpButton(msg), 0, 4)
-        grid.addWidget(self.server_host, 0, 2, 1, 2)
-        grid.addWidget(self.server_port, 0, 3)
+        self.autoconnect_cb.setToolTip(msg)
 
         label = _('Active Servers') if network.is_connected() else _('Default Servers')
-        self.servers_list_widget = QTreeWidget(parent)
+        self.servers_list_widget = QTreeWidget()
         self.servers_list_widget.setHeaderLabels( [ label, _('Limit') ] )
         self.servers_list_widget.setMaximumHeight(150)
         self.servers_list_widget.setColumnWidth(0, 240)
 
         self.change_server(host, protocol)
         self.set_protocol(protocol)
-
         self.servers_list_widget.connect(self.servers_list_widget,
                                          SIGNAL('currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)'),
                                          lambda x,y: self.server_changed(x))
-        grid.addWidget(self.servers_list_widget, 1, 1, 1, 3)
+        grid.addWidget(self.servers_list_widget, 2, 1, 1, 3)
 
         def enable_set_server():
             if config.is_modifiable('server'):
@@ -162,11 +175,10 @@ class NetworkDialog(QDialog):
         grid.addWidget(self.proxy_mode, 4, 1)
         grid.addWidget(self.proxy_host, 4, 2)
         grid.addWidget(self.proxy_port, 4, 3)
+        self.layout_ = vbox
 
-        # buttons
-        vbox.addLayout(Buttons(CancelButton(self), OkButton(self)))
-        self.setLayout(vbox)
-
+    def layout(self):
+        return self.layout_
 
     def init_servers_list(self):
         self.servers_list_widget.clear()
@@ -218,15 +230,15 @@ class NetworkDialog(QDialog):
         self.server_port.setText( port )
         self.ssl_cb.setChecked(protocol=='s')
 
-
-    def do_exec(self):
-
-        if not self.exec_():
-            return
-
-        host = str( self.server_host.text() )
-        port = str( self.server_port.text() )
+    def accept(self):
+        host = str(self.server_host.text())
+        port = str(self.server_port.text())
         protocol = 's' if self.ssl_cb.isChecked() else 't'
+        # sanitize
+        try:
+            deserialize_server(serialize_server(host, port, protocol))
+        except:
+            return
 
         if self.proxy_mode.currentText() != 'NONE':
             proxy = { 'mode':str(self.proxy_mode.currentText()).lower(),
@@ -238,4 +250,3 @@ class NetworkDialog(QDialog):
         auto_connect = self.autoconnect_cb.isChecked()
 
         self.network.set_parameters(host, port, protocol, proxy, auto_connect)
-        return True
